/***************************************************************************
                          actions.cpp  -  description
                             -------------------
    begin                : St ?ec 17 2002
    copyright            : (C) 2002 by Lumir Vanek
    email                : lvanek@users.sourceforge.net 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "actions.h"

// include files for QT
#include <qlabel.h>
#include <qwhatsthis.h>
#include <qdragobject.h>
#include <qcombobox.h>
#include <qapp.h>

// include files for KDE
#include <ktoolbar.h>
#include <kmimetype.h>
#include <kdebug.h>
#include <kiconloader.h>

//-------------------------------------------------------------------
//
// KComboAction member functions
//
//-------------------------------------------------------------------

KComboAction::KComboAction(const QString& text,
                                             int accel,
                                             const QObject *receiver,
                                             const char *member,
                                             QObject* parent,
                                             const char* name)
    : KAction(text, accel, parent, name),
      m_pCombo(0)
{
	m_receiver = receiver;
  m_member = member;
}

KComboAction::~KComboAction()
{
	//delete m_pCombo; // L.V. this cause crash !!!
}

int KComboAction::plug(QWidget *w, int index)
{
  //  if ( !w->inherits( "KToolBar" ) );
  //    return -1;

  KToolBar *toolBar = (KToolBar *) w;

  int id = KAction::getToolButtonID();
  //kdDebug() << "KComboAction::plug id=" << id << endl;

  m_pCombo = new QComboBox( toolBar, "Path Combo" );
  m_pCombo->setEditable(true);
  m_pCombo->setInsertionPolicy(QComboBox::NoInsertion);
  toolBar->insertWidget( id, 70, m_pCombo, index );
  connect( m_pCombo, SIGNAL(activated(const QString&)), m_receiver, m_member );

  addContainer( toolBar, id );

  connect(toolBar, SIGNAL(destroyed()), this, SLOT(slotDestroyed()));

  toolBar->setItemAutoSized(id, true);

  QWhatsThis::add( m_pCombo, whatsThis() );

  return containerCount() - 1;
}

void KComboAction::unplug(QWidget *w)
{
//  if ( !w->inherits( "KToolBar" ) )
//    return;

  KToolBar *toolBar = (KToolBar *)w;

  int idx = findContainer( w );
  //kdDebug() << "KComboAction::unplug idx=" << idx << " menuId=" << menuId(idx) << endl;

  toolBar->removeItem( menuId( idx ) );

  removeContainer( idx );
  m_pCombo = 0;
}

void KComboAction::slotClear()
{
  if ( containerCount() == 0 )
  {
    kdWarning() << "[KComboAction::slotClear] action not plugged" << endl;
    return;
  }

  m_pCombo->clear();
}

void KComboAction::slotClearEdit()
{
  if ( containerCount() == 0 )
  {
    kdWarning() << "[KComboAction::slotClearEdit] action not plugged" << endl;
    return;
  }

  m_pCombo->clearEdit();
}

void KComboAction::slotFocusEdit()
{
  if ( containerCount() == 0 )
  {
    kdWarning() << "[KComboAction::slotFocusEdit] action not plugged" << endl;
    return;
  }

  m_pCombo->setFocus();
}

QString KComboAction::currentText() const
{
  if ( containerCount() == 0 )
  {
    kdWarning() << "[KComboAction::currentText] action not plugged" << endl;
    return QString::null;
  }

  return m_pCombo->currentText();
}

const QPixmap * KComboAction::currentPixmap() const
{
  if ( containerCount() == 0 )
  {
    kdWarning() << "[KComboAction::currentPixmap] action not plugged" << endl;
    return 0;
  }

  return m_pCombo->pixmap( m_pCombo->currentItem() );
}

void KComboAction::insertItem( const QPixmap & pixmap, const QString & text )
{
  if ( containerCount() == 0 )
  {
    kdWarning() << "[KComboAction::insertItem] action not plugged" << endl;
    return;
  }

	if ( text.isEmpty() )
    kdWarning() << "[KComboAction::insertItem] empty string as parameter" << endl;

  int nIndex = findItem(text);
  if ( nIndex != -1 )
    m_pCombo->removeItem(nIndex);

  m_pCombo->insertItem( pixmap, text, 0 );
  m_pCombo->setCurrentItem(0);

  if ( m_pCombo->count() > 5 )
    m_pCombo->removeItem(5);
}

void KComboAction::removeItem( const QString & text )
{
  if ( containerCount() == 0 )
  {
    kdWarning() << "[KComboAction::removeItem] action not plugged" << endl;
    return;
  }

  int nIndex = findItem(text);
  if ( nIndex == -1 )
  {
    kdDebug() << "KComboAction::removeItem] item not found" << endl;
    return;
  }

  m_pCombo->removeItem(nIndex);
}

int KComboAction::findItem( const QString & text )
{
  if ( containerCount() == 0 )
  {
    kdWarning() << "[KComboAction::findItem] action not plugged" << endl;
    return -1;
  }

  int nIndex = -1;
  int i=0;
  while ( ( i < m_pCombo->count() ) && ( nIndex == -1 ) )
  {
    if ( m_pCombo->text(i) == text )
      nIndex = i;
    i++;
  }

  return nIndex;
}

/////////////////
class KDraggableLabel : public QLabel
{
public:
    KDraggableLabel( QWidget * pDragSource, const QString & text, QWidget * parent = 0, const char * name = 0)
        : QLabel(text, parent, "kde toolbar widget" )	// Use this name for it to be styled!
        , m_pDragSource(pDragSource)
    { validDrag = false;
      setAlignment( (QApplication::reverseLayout() ? Qt::AlignRight : Qt::AlignLeft) |
                 Qt::AlignVCenter | Qt::ShowPrefix );
      adjustSize();

    }

protected:
    void mousePressEvent( QMouseEvent * ev )
    	{
        validDrag = true;
        startDragPos = ev->pos();
    	}
	void mouseMoveEvent( QMouseEvent * ev )
	{
		if((startDragPos - ev->pos()).manhattanLength() > QApplication::startDragDistance())
		{	validDrag = false;

			if ( ! text().isEmpty() )
			{
				QDragObject *pDragObject = new QTextDrag( text(), m_pDragSource );
				pDragObject->setPixmap(KMimeType::pixmapForURL("text/plain", 0, KIcon::Small));
				(void) pDragObject->drag();
			}

		}
	}
    void mouseReleaseEvent( QMouseEvent * )
    {
        validDrag = false;
    }
private:
    QPoint startDragPos;
    bool validDrag;
    QWidget * m_pDragSource;
};

//-------------------------------------------------------------------
//
// KLabelAction member functions
//
//-------------------------------------------------------------------

KLabelAction::KLabelAction( const QString &text, QObject *parent, const char *name )
    : KAction( text, 0, parent, name ), m_label( 0L )
{
}

int KLabelAction::plug( QWidget *widget, int index )
{
  //do not call the previous implementation here

  if ( widget->inherits( "KToolBar" ) )
  {
    KToolBar *tb = (KToolBar *)widget;

    int id = KAction::getToolButtonID();

    m_label = new KDraggableLabel( tb->mainWindow(), text(), widget);
    tb->insertWidget( id, m_label->width(), m_label, index );

    addContainer( tb, id );

    connect( tb, SIGNAL( destroyed() ), this, SLOT( slotDestroyed() ) );

    return containerCount() - 1;
  }

  return -1;
}

void KLabelAction::unplug( QWidget *widget )
{
  if ( widget->inherits("KToolBar"))
  {
    KToolBar *bar = (KToolBar *)widget;

    int idx = findContainer( bar );

    if ( idx != -1 )
    {
      bar->removeItem( menuId( idx ) );
      removeContainer( idx );
    }

    m_label = 0;
    return;
  }
}
