/***************************************************************************
                          snmpbrowser.cpp  -  description
                             -------------------
    begin                : Po ec  8 10:15:40 CEST 2002
    copyright            : (C) 2002 by Lumir Vanek
    email                : lvanek@users.sourceforge.net 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qpainter.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <kkeydialog.h>
#include <kedittoolbar.h>
#include <kpopupmenu.h>

// include files for Net-SNMP
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>

// application specific includes
#include "snmpbrowser.h"
#include "snmpbrowserview.h"
#include "snmpbrowserdoc.h"
#include "actions.h"
#include "snmpsession.h"

#define ID_STATUS_MSG 1

SNMPBrowserApp::SNMPBrowserApp(QWidget* , const char* name):KMainWindow(0, name)
{
  config=kapp->config();

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initStatusBar();
  initActions();
  initDocument();
  initView();
	
  readOptions();

  ///////////////////////////////////////////////////////////////////
  // disable actions at startup
  fileDisconnect->setEnabled(false);
  //fileSave->setEnabled(false);
  //fileSaveAs->setEnabled(false);
  //filePrint->setEnabled(false);
  editCut->setEnabled(false);
  editCopy->setEnabled(false);
  editPaste->setEnabled(false);

  viewExpandNode->setEnabled(false);
	viewCollapseNode->setEnabled(false);

	init_snmp("SNMPBrowser"); // initialize SNMP library
}

SNMPBrowserApp::~SNMPBrowserApp()
{

}

void SNMPBrowserApp::initActions()
{
  //-----------------------------------------------------------------
	// File menu

  fileConnect = new KAction(i18n("&Connect..."), "connect_established", CTRL+Key_T, this, SLOT(slotConnect()), actionCollection(), "connect_established");
	fileDisconnect = new KAction(i18n("&Disconnect"), "connect_no", CTRL+Key_D, this, SLOT(slotDisconnect()), actionCollection(), "connect_no");

  //fileNewWindow = new KAction(i18n("New &Window"), 0, 0, this, SLOT(slotFileNewWindow()), actionCollection(),"file_new_window");
  //fileNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
  //fileOpen = KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());
  //fileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection());
  //fileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection());
  //fileSaveAs = KStdAction::saveAs(this, SLOT(slotFileSaveAs()), actionCollection());
  //fileClose = KStdAction::close(this, SLOT(slotFileClose()), actionCollection());
  //filePrint = KStdAction::print(this, SLOT(slotFilePrint()), actionCollection());
  fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());
  editCut = KStdAction::cut(this, SLOT(slotEditCut()), actionCollection());
  editCopy = KStdAction::copy(this, SLOT(slotEditCopy()), actionCollection());
  editPaste = KStdAction::paste(this, SLOT(slotEditPaste()), actionCollection());

  //-----------------------------------------------------------------
	// View menu
  viewExpandNode = new KToolBarPopupAction(i18n("&Expand Node"), "expand_node", CTRL+Key_Plus, this, SLOT(slotExpandNode()), actionCollection(), "expand_node");
	KPopupMenu * pMenuExpNode = viewExpandNode->popupMenu();
	connect(pMenuExpNode, SIGNAL(activated(int)), this, SLOT(slotExpandNodeActivated(int)));
	for (uint i = 1; i <= 8; i++)
		pMenuExpNode->insertItem( i18n("Expand to level %1").arg(i), i+1 );

	viewCollapseNode= new KToolBarPopupAction(i18n("&Collapse Node"), "collapse_node", CTRL+Key_Minus, this, SLOT(slotCollapseNode()), actionCollection(), "collapse_node");
	KPopupMenu * pMenuColNode = viewCollapseNode->popupMenu();
	connect(pMenuColNode, SIGNAL(activated(int)), this, SLOT(slotCollapseNodeActivated(int)) );
	for (uint i = 1; i <= 8; i++)
		pMenuColNode->insertItem( i18n("Collapse to level %1").arg(i), i+1 );


  //-----------------------------------------------------------------
	// Settings menu
	viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection(), "showtoolbar");
  viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection(), "showstatusbar");
	viewPathBar = new KToggleAction( i18n( "Show &Path Toolbar" ), 0, this, SLOT( slotShowLocationBar() ), actionCollection(), "showlocationbar" );
  KStdAction::keyBindings(this, SLOT(slotConfigureKeys()), actionCollection(), "configurekeys");
  KStdAction::configureToolbars(this, SLOT(slotConfigToolbars()), actionCollection(), "configuretoolbars");
	//KStdAction::preferences(this, SLOT (slotConfigure()), actionCollection(), "configure");

	//-----------------------------------------------------------------
	
  //fileNewWindow->setStatusText(i18n("Opens a new application window"));
  //fileNew->setStatusText(i18n("Creates a new document"));
  //fileOpen->setStatusText(i18n("Opens an existing document"));
  //fileOpenRecent->setStatusText(i18n("Opens a recently used file"));
  //fileSave->setStatusText(i18n("Saves the actual document"));
  //fileSaveAs->setStatusText(i18n("Saves the actual document as..."));
  //fileClose->setStatusText(i18n("Closes the actual document"));
  //filePrint ->setStatusText(i18n("Prints out the actual document"));
  fileQuit->setStatusText(i18n("Quits the application"));
  editCut->setStatusText(i18n("Cuts the selected section and puts it to the clipboard"));
  editCopy->setStatusText(i18n("Copies the selected section to the clipboard"));
  editPaste->setStatusText(i18n("Pastes the clipboard contents to actual position"));
  viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
  viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));

  //-----------------------------------------------------------------
  // Path bar initialization - based on Konqueror: KonqMainWindow::initActions()
  new KAction( i18n("Clear Path bar"), BarIcon("locationbar_erase", 16), 0, this, SLOT(slotClearLocationBar()), actionCollection(), "path_clear" );
  new KLabelAction( i18n("&Path "), actionCollection(), "path_label");
  pathCombo = new KComboAction( i18n("Path Bar"), 0, this, SLOT(slotPathComboActivated(const QString &)), actionCollection(), "path_combo");

  // use the absolute path to your snmpbrowserui.rc file for testing purpose in createGUI();
  createGUI(); // "/home/lvanek/Projects/snmpbrowser/snmpbrowser/snmpbrowserui.rc"

}


void SNMPBrowserApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG);
}

void SNMPBrowserApp::initDocument()
{
  doc = new SNMPBrowserDoc(this);
  doc->newDocument();
}

void SNMPBrowserApp::initView()
{ 
  ////////////////////////////////////////////////////////////////////
  // create the main widget here that is managed by KTMainWindow's view-region and
  // connect the widget to your document to display document contents.

  view = new SNMPBrowserView(this);
  doc->addView(view);

  // SNMPBrowserView is derived from QSplitter
  view->setOrientation(Horizontal);
	view->setOpaqueResize(TRUE);
  connect(view, SIGNAL(sigSelectedElementChanged(const SnmpTreeItem *)), this, SLOT(slotSelectedElementChanged(const SnmpTreeItem *)) );


  setCentralWidget(view);	
  setCaption(doc->URL().fileName(),false);

}

void SNMPBrowserApp::openDocumentFile(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));

  doc->openDocument( url);
  //fileOpenRecent->addURL( url );
  slotStatusMsg(i18n("Ready."));
}


SNMPBrowserDoc *SNMPBrowserApp::getDocument() const
{
  return doc;
}

void SNMPBrowserApp::saveOptions()
{	
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", viewToolBar->isChecked());
  config->writeEntry("Show Path Bar", viewPathBar->isChecked());
   config->writeEntry("Show Statusbar",viewStatusBar->isChecked());
  config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
  config->writeEntry("ToolBarPos_pathToolBar", (int) toolBar("pathToolBar")->barPos());
  
  //fileOpenRecent->saveEntries(config,"Recent Files");
}


void SNMPBrowserApp::readOptions()
{
	
  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  viewToolBar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();

  bViewToolbar = config->readBoolEntry("Show Path Bar", true);
  viewPathBar->setChecked(bViewToolbar);
  slotShowLocationBar();

  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);

  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos_pathToolBar", KToolBar::Top);
  toolBar("pathToolBar")->setBarPos(toolBarPos);
	
  // initialize the recent file list
  //fileOpenRecent->loadEntries(config,"Recent Files");

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }
}

void SNMPBrowserApp::saveProperties(KConfig *_cfg)
{
  if(doc->URL().fileName()!=i18n("Untitled") && !doc->isModified())
  {
    // saving to tempfile not necessary

  }
  else
  {
    KURL url=doc->URL();	
    _cfg->writeEntry("filename", url.url());
    _cfg->writeEntry("modified", doc->isModified());
    QString tempname = kapp->tempSaveName(url.url());
    QString tempurl= KURL::encode_string(tempname);
    KURL _url(tempurl);
    doc->saveDocument(_url);
  }
}


void SNMPBrowserApp::readProperties(KConfig* _cfg)
{
  QString filename = _cfg->readEntry("filename", "");
  KURL url(filename);
  bool modified = _cfg->readBoolEntry("modified", false);
  if(modified)
  {
    bool canRecover;
    QString tempname = kapp->checkRecoverFile(filename, canRecover);
    KURL _url(tempname);
  	
    if(canRecover)
    {
      doc->openDocument(_url);
      doc->setModified();
      setCaption(_url.fileName(),true);
      QFile::remove(tempname);
    }
  }
  else
  {
    if(!filename.isEmpty())
    {
      doc->openDocument(url);
      setCaption(url.fileName(),false);
    }
  }
}		

bool SNMPBrowserApp::queryClose()
{
  return doc->saveModified();
}

bool SNMPBrowserApp::queryExit()
{
  saveOptions();
  return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void SNMPBrowserApp::slotFileNewWindow()
{
  slotStatusMsg(i18n("Opening a new application window..."));
	
  SNMPBrowserApp *new_window= new SNMPBrowserApp();
  new_window->show();

  slotStatusMsg(i18n("Ready."));
}

void SNMPBrowserApp::slotFileNew()
{
  slotStatusMsg(i18n("Creating new document..."));

  if(!doc->saveModified())
  {
     // here saving wasn't successful

  }
  else
  {	
    doc->newDocument();		
    setCaption(doc->URL().fileName(), false);
  }

  slotStatusMsg(i18n("Ready."));
}

void SNMPBrowserApp::slotFileOpen()
{
  slotStatusMsg(i18n("Opening file..."));
	
  if(!doc->saveModified())
  {
     // here saving wasn't successful

  }
  else
  {	
    KURL url=KFileDialog::getOpenURL(QString::null,
        i18n("*|All files"), this, i18n("Open File..."));
    if(!url.isEmpty())
    {
      doc->openDocument(url);
      setCaption(url.fileName(), false);
      //fileOpenRecent->addURL( url );
    }
  }
  slotStatusMsg(i18n("Ready."));
}

void SNMPBrowserApp::slotFileOpenRecent(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));
	
  if(!doc->saveModified())
  {
     // here saving wasn't successful
  }
  else
  {
    doc->openDocument(url);
    setCaption(url.fileName(), false);
  }

  slotStatusMsg(i18n("Ready."));
}

void SNMPBrowserApp::slotFileSave()
{
  slotStatusMsg(i18n("Saving file..."));
	
  doc->saveDocument(doc->URL());

  slotStatusMsg(i18n("Ready."));
}

void SNMPBrowserApp::slotFileSaveAs()
{
  slotStatusMsg(i18n("Saving file with a new filename..."));

  KURL url=KFileDialog::getSaveURL(QDir::currentDirPath(),
        i18n("*|All files"), this, i18n("Save as..."));
  if(!url.isEmpty())
  {
    doc->saveDocument(url);
    //fileOpenRecent->addURL(url);
    setCaption(url.fileName(),doc->isModified());
  }

  slotStatusMsg(i18n("Ready."));
}

void SNMPBrowserApp::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));
	
  close();

  slotStatusMsg(i18n("Ready."));
}

void SNMPBrowserApp::slotFilePrint()
{
  slotStatusMsg(i18n("Printing..."));

  QPrinter printer;
  if (printer.setup(this))
  {
    view->print(&printer);
  }

  slotStatusMsg(i18n("Ready."));
}

void SNMPBrowserApp::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  saveOptions();
  // close the first window, the list makes the next one the first again.
  // This ensures that queryClose() is called on each window to ask for closing
  KMainWindow* w;
  if(memberList)
  {
    for(w=memberList->first(); w!=0; w=memberList->first())
    {
      // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
      // the window and the application stay open.
      if(!w->close())
	break;
    }
  }	
}

void SNMPBrowserApp::slotEditCut()
{
  slotStatusMsg(i18n("Cutting selection..."));

  slotStatusMsg(i18n("Ready."));
}

void SNMPBrowserApp::slotEditCopy()
{
  slotStatusMsg(i18n("Copying selection to clipboard..."));

  slotStatusMsg(i18n("Ready."));
}

void SNMPBrowserApp::slotEditPaste()
{
  slotStatusMsg(i18n("Inserting clipboard contents..."));

  slotStatusMsg(i18n("Ready."));
}

void SNMPBrowserApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggling toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!viewToolBar->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }		

  slotStatusMsg(i18n("Ready."));
}

void SNMPBrowserApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}


void SNMPBrowserApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}

/** Configure key bindings */
void SNMPBrowserApp::slotConfigureKeys()
{
	KKeyDialog::configureKeys(actionCollection(), xmlFile());
}

/** Configure application toolbars */
void SNMPBrowserApp::slotConfigToolbars()
{
	saveMainWindowSettings(KGlobal::config(), "SNMPBrowserMainWindow");
  QString savedPath = pathCombo->currentText();
	const QPixmap * pTmp = pathCombo->currentPixmap();
  QPixmap savedPixmap = ( pTmp ? QPixmap(*pTmp) : QPixmap() );
	KEditToolbar dlg(actionCollection());

	if (dlg.exec())
		{ createGUI();

			if( ! savedPath.isEmpty() )
     		pathCombo->insertItem( savedPixmap, savedPath );
		}
}

/** Connect to SNMP device */
void SNMPBrowserApp::slotConnect()
{
	slotStatusMsg(i18n("Create new connection to SNMP device ..."));
	view->Connect();
	slotStatusMsg(i18n("Ready."));
}

/** Disconnect from SNMP device */
void SNMPBrowserApp::slotDisconnect()
{
	slotStatusMsg(i18n("Remove existing connection to SNMP device ..."));
	view->Disconnect();
	slotStatusMsg(i18n("Ready."));
}

/** Called when Combo path is activated */
void SNMPBrowserApp::slotPathComboActivated(const QString &strPath)
{
	if(! view->changePath(strPath))
		pathCombo->removeItem(strPath); // path not found, remove it from combo
}

/** Clear location bar */
void SNMPBrowserApp::slotClearLocationBar()
{
	pathCombo->slotClearEdit();
	pathCombo->slotFocusEdit();
	view->slotUnselect();
}
/** Called when user change tree item  */
void SNMPBrowserApp::slotSelectedElementChanged( const SnmpTreeItem *pSnmpTreeItem)
{
	/*m_Actions.editCut->setEnabled(true);
  m_Actions.editCopy->setEnabled(true);
  m_Actions.editPaste->setEnabled(true);

  m_Actions.goUp->setEnabled(true);

  m_Actions.bookmarksToggle->setEnabled(true);*/

  SnmpTreeItem::enItemType type = pSnmpTreeItem->itemType();

	fileDisconnect->setEnabled(type == SnmpTreeItem::itemSession);
	
	pathCombo->insertItem(pSnmpTreeItem->pixmap(0) ? QPixmap(*pSnmpTreeItem->pixmap(0)) : QPixmap(), pSnmpTreeItem->path(0) );

	viewExpandNode->setEnabled(pSnmpTreeItem->childCount() != 0);
	viewCollapseNode->setEnabled(pSnmpTreeItem->childCount() != 0);
}

/** Called when menu item from Expand Node button is selected */
void SNMPBrowserApp::slotExpandNodeActivated(int id)
{
	view->expandNode(id);
}

/** Called when menu item from Collapse Node button is selected */
void SNMPBrowserApp::slotCollapseNodeActivated(int id)
{
	view->collapseNode(id);
}
/** Expand tree item subtree */
void SNMPBrowserApp::slotExpandNode()
{
	view->expandNode(-1);
}

/** Collapse tree item subtree */
void SNMPBrowserApp::slotCollapseNode()
{
	view->collapseNode(-1);
}

/** Display/hide path bar */
void SNMPBrowserApp::slotShowLocationBar()
{
	slotStatusMsg(i18n("Toggling pathbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!viewPathBar->isChecked())
  {
    toolBar("pathToolBar")->hide();
  }
  else
  {
    toolBar("pathToolBar")->show();
  }

  slotStatusMsg(i18n("Ready."));
}
