/***************************************************************************
                          snmpbrowserview.cpp  -  description
                             -------------------
    begin                : Po ec  8 10:15:40 CEST 2002
    copyright            : (C) 2002 by Lumir Vanek
    email                : lvanek@users.sourceforge.net 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qprinter.h>
#include <qpainter.h>

// include files for KDE
#include <keditcl.h>
#include <klistview.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kmessagebox.h>

// application specific includes
#include "snmpbrowserview.h"
#include "snmpbrowserdoc.h"
#include "snmpbrowser.h"
#include "connectdlg.h"
#include "snmpsession.h"


QPixmap *g_pPixmapSession = 0;
QPixmap *g_pPixmapFolder = 0;
QPixmap *g_pPixmapVariable = 0;


SNMPBrowserView::SNMPBrowserView(QWidget *parent, const char *name) : QSplitter(parent, name)
{
  setBackgroundMode(PaletteBase);

  m_pTreeView = new KListView(this, "Tree view");
  m_pTreeView->addColumn(i18n("Tree"));

	connect(m_pTreeView, SIGNAL(selectionChanged(QListViewItem *)), this, SLOT(slotETreeViewSelChanged(QListViewItem *)));
	  
  m_pTreeView->setSelectionMode(QListView::Single);

  //if(OptionsDialog::getTreeViewRootDecorated())
  	m_pTreeView->setRootIsDecorated(true);

  m_pEditView = new KEdit(this, "Edit view");

  g_pPixmapSession = new QPixmap(SmallIcon("socket"));
  g_pPixmapVariable = new QPixmap(SmallIcon("blockdevice"));
  g_pPixmapFolder = new QPixmap(SmallIcon("folder"));
}

SNMPBrowserView::~SNMPBrowserView()
{
	if(g_pPixmapSession)
		delete g_pPixmapSession;

	if(g_pPixmapVariable)
		delete g_pPixmapVariable;

	if(g_pPixmapFolder)
		delete g_pPixmapFolder;
}

SNMPBrowserDoc *SNMPBrowserView::getDocument() const
{
  SNMPBrowserApp *theApp=(SNMPBrowserApp *) parentWidget();

  return theApp->getDocument();
}

void SNMPBrowserView::print(QPrinter *pPrinter)
{
  QPainter printpainter;
  printpainter.begin(pPrinter);
	
  // TODO: add your printing code here

  printpainter.end();
}

/** Connect to new SNMP device */
void SNMPBrowserView::Connect()
{
	ConnectDlg dlg(this, "Connect");
	if(dlg.exec() == ConnectDlg::Accepted)
		{ if(rootTreeItem(dlg.m_strHostNameOrIP))
				{ KMessageBox::sorry(this, i18n("Session with this name already exists !"));
					return;
				}
				
			SnmpSession *pSnmpSession = new SnmpSession(m_pTreeView,
																									dlg.m_strConnectionName,
																									dlg.m_strHostNameOrIP,
																									dlg.m_strCommunity,
																									dlg.m_iPort);
			if(pSnmpSession->SnmpOpen() == false)
				{ delete pSnmpSession;
					KMessageBox::sorry(this, i18n("Session open failed !"));
					return;
				}

			// Read OIDs tree
			QStringList strListOfPaths;
			pSnmpSession->SnmpWalk(strListOfPaths);

			QString strPath;

			for(QStringList::Iterator it = strListOfPaths.begin(); it != strListOfPaths.end(); ++it)
				{ strPath = *it;
					SnmpTreeItem *pSnmpTreeItem = InsertItem(pSnmpSession, strPath);

					// Read OID
					if(pSnmpTreeItem && pSnmpTreeItem->itemType() == SnmpTreeItem::itemVariable)
						{ static_cast <SnmpVariable *> (pSnmpTreeItem)->Read_objid(strPath);
						}
				}
		}
}

/** Disconnect from SNMP device */
void SNMPBrowserView::Disconnect()
{
	// get selected item from tree view
	QListViewItem *pSelectedItem = m_pTreeView->selectedItem();
	if(!pSelectedItem)
		{ KMessageBox::sorry(this, i18n("Device not selected !"));
			return;
		}

	SnmpTreeItem* pSnmpTreeItem = static_cast<SnmpTreeItem *> (pSelectedItem);
	if(pSnmpTreeItem && pSnmpTreeItem->itemType() == SnmpTreeItem::itemSession)
		{ // Close and delete session
			delete pSnmpTreeItem; // destructor closes session
		}
	else
		KMessageBox::sorry(this, i18n("Device not selected !"));
}

/** Insert new item into tree */
SnmpTreeItem* SNMPBrowserView::InsertItem(QListViewItem *pParentItem, const QString &strPath)
{
	QString strTmpPath(strPath);
	QString strItemName;

	bool bEnd = false;
	do  { strItemName = getItemNameFromPath(strTmpPath);
				if(strTmpPath.find('.') < 0)
					{ bEnd = true;
						strItemName += " (" + strTmpPath + ")";
					}	
				
				if(!bEnd)
					{	// find folder name in childs
						bool bFound = false;
					
						QListViewItem *pChild = pParentItem->firstChild();
						while(pChild)
						{ if(pChild->text(0) == strItemName)
								{ bFound = true;
									pParentItem = pChild;
									break;
								}
      
							pChild = pChild->nextSibling();
						}

						// if folder not exists, create it as new parent
						if(!bFound)
							{ pParentItem = new SnmpFolder(pParentItem, strItemName);
							}
						
					}
				else
					{ // create new item
					  SnmpTreeItem *pLvItemFolder = new SnmpVariable(pParentItem, strItemName);
						pLvItemFolder->setPixmap(0, *g_pPixmapVariable);
						return pLvItemFolder;
					}

			} while(bEnd == false);

	return 0;		
}

/** Extract one item name from full path */
QString SNMPBrowserView::getItemNameFromPath(QString &strPath)
{
	QString strItemName;

	int iIndex = strPath.find('.');
	if(iIndex > 0)
		{ strItemName = strPath.left(iIndex);
			strPath.remove(0, iIndex + 1);
			return strItemName;
		}
	return strPath;
}

/** Called when user change selected tree item */
void SNMPBrowserView::slotETreeViewSelChanged(QListViewItem *item)
{
	ASSERT(item);
	SnmpTreeItem* pSnmpTreeItem = static_cast<SnmpTreeItem *> (item);

	if(pSnmpTreeItem && pSnmpTreeItem->itemType() == SnmpTreeItem::itemVariable)
		{ // Find parent session
			QListViewItem *pParentItem = 0;
			QListViewItem *pCurrentItem = item;
			bool bFound = false;
			do
				{ pParentItem = pCurrentItem->parent();
					if(static_cast <SnmpTreeItem *> (pParentItem)->itemType() == SnmpTreeItem::itemSession)
						bFound = true;

					pCurrentItem = pParentItem;
					
				} while(!bFound && pParentItem);
				
			QString strVarible;
			static_cast <SnmpSession *> (pParentItem)->SnmpGet(static_cast<SnmpVariable *>(pSnmpTreeItem)->m_OidItem.m_anOID,
																												 static_cast<SnmpVariable *>(pSnmpTreeItem)->m_OidItem.m_anOID_len,
																												 strVarible);
			
			
		  m_pEditView->setText(strVarible);
		}
	else
		m_pEditView->setText("");

	emit sigSelectedElementChanged(pSnmpTreeItem);
}

/** Find item with strPath and select it */
bool SNMPBrowserView::changePath(const QString &strPath)
{
	QString strTmpPath(strPath);
	QString strItemName = getItemNameFromPath(strTmpPath);
	SnmpTreeItem* pRootItem = rootTreeItem(strItemName);
	if(!pRootItem)
		return false;

	SnmpTreeItem *pFoundItem = pRootItem->searchPath(strPath);
	if(pFoundItem)
		{ // make new item visible and selected
			m_pTreeView->setCurrentItem(pFoundItem);
			m_pTreeView->ensureItemVisible(pFoundItem);
			return true;
		}
	return false;
}

/** Return pointer to root SNMP device with given name */
SnmpTreeItem* SNMPBrowserView::rootTreeItem(const QString &strText)
{
	SnmpTreeItem* pTreeItem = static_cast <SnmpTreeItem *> (m_pTreeView->firstChild());
	while(pTreeItem)
  		{ if((pTreeItem->itemType() == SnmpTreeItem::itemSession) && (pTreeItem->text(0) == strText))
  				{ return static_cast <SnmpTreeItem *> (pTreeItem);
  				}
  			pTreeItem = static_cast <SnmpTreeItem *> (pTreeItem->nextSibling());
  		}
	return 0L;
}

/** Expand selected node and entire child subtree  */
void SNMPBrowserView::expandNode(int iLevel)
{
	// get selected item from tree view
	QListViewItem *pSelectedItem = m_pTreeView->selectedItem();
	if(!pSelectedItem)
		{ KMessageBox::sorry(this, i18n("Tree item not selected !"));
			return;
		}

	// expand node
	SnmpTreeItem* pSelectedTreeItem = static_cast <SnmpTreeItem *> (pSelectedItem);
	pSelectedTreeItem->expandNode(iLevel);
}

/** Collapse selected item subtree */
void SNMPBrowserView::collapseNode(int iLevel)
{
	// get selected item from tree view
	QListViewItem *pSelectedItem = m_pTreeView->selectedItem();
	if(!pSelectedItem)
		{ KMessageBox::sorry(this, i18n("Tree item not selected !"));
			return;
		}

	// collapse node
	SnmpTreeItem* pSelectedTreeItem = static_cast <SnmpTreeItem *> (pSelectedItem);
	pSelectedTreeItem->collapseNode(iLevel);
}

/** Unselect the currently selected item */
void SNMPBrowserView::slotUnselect()
{
	m_pTreeView->selectAll(false);
	m_pEditView->clear();
}
