/***************************************************************************
                          snmpbrowserview.h  -  description
                             -------------------
    begin                : Po ec  8 10:15:40 CEST 2002
    copyright            : (C) 2002 by Lumir Vanek
    email                : lvanek@users.sourceforge.net 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SNMPBROWSERVIEW_H
#define SNMPBROWSERVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for Qt
#include <qsplitter.h>

class SNMPBrowserDoc;
class KListView;
class KEdit;
class QListViewItem;
class QString;
class SnmpTreeItem;

/** The SNMPBrowserView class provides the view widget for the SNMPBrowserApp instance.	
 * The View instance inherits QWidget as a base class and represents the view object of a KTMainWindow. As SNMPBrowserView is part of the
 * docuement-view model, it needs a reference to the document object connected with it by the SNMPBrowserApp class to manipulate and display
 * the document structure provided by the SNMPBrowserDoc class.
 * 	
 * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
 * @version KDevelop version 0.4 code generation
 */
class SNMPBrowserView : public QSplitter
{
  Q_OBJECT
  public:
    /** Constructor for the main view */
    SNMPBrowserView(QWidget *parent = 0, const char *name=0);
    /** Destructor for the main view */
    ~SNMPBrowserView();

    /** returns a pointer to the document connected to the view instance. Mind that this method requires a SNMPBrowserApp instance as a parent
     * widget to get to the window document pointer by calling the SNMPBrowserApp::getDocument() method.
     *
     * @see SNMPBrowserApp#getDocument
     */
    SNMPBrowserDoc *getDocument() const;

    /** contains the implementation for printing functionality */
    void print(QPrinter *pPrinter);

    /** Disconnect from SNMP device */
		void Disconnect();

		/** Connect to new SNMP device */
		void Connect();
		
  /** Extract one item name from full path */
  QString getItemNameFromPath(QString &);
  
  /** Find item with strPath and select it */
  bool changePath(const QString &);
  
	 /** Expand selected node and entire child subtree  */
  void expandNode(int);

  /** Collapse selected item subtree */
  void collapseNode(int);

  /** Return pointer to root SNMP device with given name */
	SnmpTreeItem* rootTreeItem(const QString &);
  
protected: // Protected attributes
  /** Left view with SNMP data structure */
  KListView * m_pTreeView;

  /** Edit vidget with detailed info about selected tree item */
  KEdit * m_pEditView;

public slots: // Public slots
  
protected: // Protected methods
  /** Insert new item into tree */
  SnmpTreeItem * InsertItem(QListViewItem *, const QString &);
protected slots: // Protected slots
  /** Called when user change selected tree item */
  void slotETreeViewSelChanged(QListViewItem *);
public slots: // Public slots
  /** Unselect the currently selected item */
  void slotUnselect();
signals: // Signals
  /** Fired, when user changed active tree item */
  void sigSelectedElementChanged(const SnmpTreeItem *);
};

#endif // SNMPBROWSERVIEW_H
