/***************************************************************************
                          snmpsession.cpp  -  description
                             -------------------
    begin                : P ?ec 12 2002
    copyright            : (C) 2002 by Lumir Vanek
    email                : lvanek@users.sourceforge.net 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "snmpsession.h"

#include <klistview.h>
#include <kmessagebox.h>
#include <klocale.h>

extern QPixmap *g_pPixmapSession;
extern QPixmap *g_pPixmapFolder;
extern QPixmap *g_pPixmapVariable;

#define NETSNMP_DS_WALK_INCLUDE_REQUESTED	1
#define NETSNMP_DS_WALK_PRINT_STATISTICS	2
#define NETSNMP_DS_WALK_DONT_CHECK_LEXICOGRAPHIC	3


//-----------------------------------------------------------------------------
//
// SnmpTreeItem implementation
//
//-----------------------------------------------------------------------------

SnmpTreeItem::SnmpTreeItem(KListView *pParent, const QString& name)
	: QListViewItem(pParent, name)
{
	//m_bBookmark = false;
}

SnmpTreeItem::SnmpTreeItem(QListViewItem *pParent, const QString& name)
	: QListViewItem(pParent, name)
{
	//m_bBookmark = false;
}

/** Return path for this XML item as string */
QString SnmpTreeItem::path(const SnmpTreeItem *) const
{
	QString strPath;

	// get parent item
	QListViewItem *pParentItem = parent();

	// first part of path is parents path
	if(pParentItem)
		{ SnmpTreeItem *pSnmpTreeItemParent = static_cast <SnmpTreeItem *> (pParentItem);
		  strPath = pSnmpTreeItemParent->path(this);
		}

	// dot separator
	if(strPath.length() > 0)
		strPath += ".";

	// its all, last part of path add derived class
	return strPath;
}

/** Search item with path from parameter */
SnmpTreeItem * SnmpTreeItem::searchPath(const QString & strPath)
{
	if(text(0) == strPath)
		return this;

	QString strTmpPath(strPath);
	QString strItemName = getItemNameFromPath(strTmpPath);

	if(text(0) != strItemName)
		return 0;

	// send searching to child with index iIndex
	SnmpTreeItem* pChildItem = (SnmpTreeItem*) firstChild();
  while(pChildItem)
		{ SnmpTreeItem *pFoundItem = pChildItem->searchPath(strTmpPath);
			if(pFoundItem)
				return pFoundItem;
			
			pChildItem = (SnmpTreeItem*) pChildItem->nextSibling();
    }
	return 0;
}

/** Extract one item name from full path */
QString SnmpTreeItem::getItemNameFromPath(QString &strPath)
{
	QString strItemName;

	int iIndex = strPath.find('.');
	if(iIndex > 0)
		{ strItemName = strPath.left(iIndex);
			strPath.remove(0, iIndex + 1);
			return strItemName;
		}
	return strPath;
}

/** Expand tree node up to level specified in parameter. /-1: expand entire node/.  */
void SnmpTreeItem::expandNode(int iLevel, int iCurrentLevel)
{
	if((iLevel > 0) && (iCurrentLevel >= iLevel))
		return;

	// expand this node
	setOpen(true);

	// expand my childrens
	SnmpTreeItem* pChildTreeItem = (SnmpTreeItem*) firstChild();
  while(pChildTreeItem)
			{ // expand child node
		  	pChildTreeItem->expandNode(iLevel, iCurrentLevel >= 0 ? iCurrentLevel + 1 : iCurrentLevel);

				pChildTreeItem = (SnmpTreeItem*) pChildTreeItem->nextSibling();
  		}
}

/** Collapse tree node from level specified in parameter. /-1: collapse entire node/.  */
void SnmpTreeItem::collapseNode(int iLevel, int iCurrentLevel)
{
	if((iLevel == -1) || (iCurrentLevel >= iLevel))
		{	// expand this node
			setOpen(false);
		}

	// collapse my childrens
	SnmpTreeItem* pChildTreeItem = (SnmpTreeItem*) firstChild();
  while(pChildTreeItem)
		{ // collapse child node
	 		pChildTreeItem->collapseNode(iLevel, iCurrentLevel >= 0 ? iCurrentLevel + 1 : iCurrentLevel);

			pChildTreeItem = (SnmpTreeItem*) pChildTreeItem->nextSibling();
  	}
}

//-----------------------------------------------------------------------------
//
// SnmpSession implementation
//
//-----------------------------------------------------------------------------

SnmpSession::SnmpSession(KListView *parent,
												 const QString &strConnectionName,
												 const QString &strHostNameOrIP,
												 const QString &strCommunity,
												 int iPort)
	: SnmpTreeItem(parent, strConnectionName)
{
	m_pSession = 0;
	
  snmp_sess_init(&m_Session);                   /* set up defaults */
  m_Session.peername = "ucd-snmp.ucdavis.edu";

  m_strCommunity = strCommunity;
	m_iPort = iPort;
	m_strHost = strHostNameOrIP;

	// Set pixmap for SNMP session
	setPixmap(0, *g_pPixmapSession);
}

SnmpSession::~SnmpSession()
{
	if(m_pSession)
		snmp_close(m_pSession);
}

/** Return path for this item as string */
QString SnmpSession::path(const SnmpTreeItem *pItemCaller) const
{
	return text(0);
}

/** Open SNMP session */
bool SnmpSession::SnmpOpen()
{
	/* set the SNMPv1 community name used for authentication */
	m_Session.community = (u_char*) m_strCommunity.data();
	m_Session.community_len = m_strCommunity.length();

	/* set the SNMP version number */
	m_Session.version = SNMP_VERSION_1;

	m_Session.peername		= m_strHost.data();
	m_Session.remote_port	= m_iPort;
	// m_Session.timeout		= _timeOut;
	// m_Session.retry = _retries;

	m_pSession = snmp_open(&m_Session);   // establish the session 
	if(m_pSession == 0)
		return false;

	return true;
}

bool SnmpSession::SnmpGet(oid *theoid, size_t theoid_len, QString &strVariable)
{
  bool   bExitval = true;
  struct snmp_pdu *pdu, *response;
  struct variable_list *vars;
  int status;

  pdu = snmp_pdu_create(SNMP_MSG_GET);
  snmp_add_null_var(pdu, theoid, theoid_len);

  status = snmp_synch_response(m_pSession, pdu, &response);
  if(status == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR)
		{ for(vars = response->variables; vars; vars = vars->next_variable)
				{ //numprinted++;
					QCString cstrTmp(SPRINT_MAX_LEN + 1);
          snprint_value(cstrTmp.data(), SPRINT_MAX_LEN, vars->name, vars->name_length, vars);
          strVariable = cstrTmp;
				}
		}
	else
		{ if(status == STAT_SUCCESS)
				KMessageBox::sorry(0, i18n("Error in packet\nReason: %s").arg(snmp_errstring(response->errstat)));
      else
        snmp_sess_perror("SNMPBrowser", m_pSession);

      bExitval = false;
		}	
		
  if(response)
    snmp_free_pdu(response);

  return bExitval;
}

#define DS_WALK_INCLUDE_REQUESTED	1
#define DS_WALK_PRINT_STATISTICS	2
#define DS_WALK_DONT_CHECK_LEXICOGRAPHIC	3

oid objid_mib[] = {1, 3, 6, 1, 2, 1};
int numprinted = 0;

/** Source borrowed from snmpwalk.c, fill string list with output */
bool SnmpSession::SnmpWalk(QStringList &strListOfPaths)
{
	struct snmp_pdu *pdu, *response;
  struct variable_list *vars;
  int    arg;
  oid    name[MAX_OID_LEN];
  size_t name_length;
  oid    root[MAX_OID_LEN];
  size_t rootlen;
  int    count;
  int    running;
  int    status;
  int    check;
  bool   bExitval = true;

  numprinted = 0;

  netsnmp_ds_register_config(ASN_BOOLEAN, "snmpbrowser", "includeRequested",
			       NETSNMP_DS_APPLICATION_ID,
			       NETSNMP_DS_WALK_INCLUDE_REQUESTED);

  netsnmp_ds_register_config(ASN_BOOLEAN, "snmpbrowser", "printStatistics",
			       NETSNMP_DS_APPLICATION_ID,
			       NETSNMP_DS_WALK_PRINT_STATISTICS);

  netsnmp_ds_register_config(ASN_BOOLEAN, "snmpbrowser", "dontCheckOrdering",
			       NETSNMP_DS_APPLICATION_ID,
			       NETSNMP_DS_WALK_DONT_CHECK_LEXICOGRAPHIC);

             
  memmove(root, objid_mib, sizeof(objid_mib));
  rootlen = sizeof(objid_mib) / sizeof(oid);


  /* get first object to start walk */
  memmove(name, root, rootlen * sizeof(oid));
  name_length = rootlen;

  running = 1;

  
  while(running)
		{ /* create PDU for GETNEXT request and add object name to request */
      pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
      snmp_add_null_var(pdu, name, name_length);

      /* do the request */
      status = snmp_synch_response(m_pSession, pdu, &response);
      if(status == STAT_SUCCESS)
				{ if (response->errstat == SNMP_ERR_NOERROR)
					 { /* check resulting variables */
						 for(vars = response->variables; vars; vars = vars->next_variable)
							 { if((vars->name_length < rootlen) || (memcmp(root, vars->name, rootlen * sizeof(oid))!=0))
									 { /* not part of this subtree */
										 running = 0;
										 continue;
									 }
								 numprinted++;

								 QCString cstrTmp(SPRINT_MAX_LEN + 1);
								 snprint_objid(cstrTmp.data(), SPRINT_MAX_LEN, vars->name, vars->name_length);
								 strListOfPaths.append(cstrTmp);

								 if((vars->type != SNMP_ENDOFMIBVIEW) && (vars->type != SNMP_NOSUCHOBJECT) && (vars->type != SNMP_NOSUCHINSTANCE))
									 { /* not an exception value */
									   if(check && snmp_oid_compare(name, name_length, vars->name, vars->name_length) >= 0)
											 { char name_buf[SPRINT_MAX_LEN], var_buf[SPRINT_MAX_LEN];
												 snprint_objid(name_buf, sizeof(name_buf), name, name_length);
												 snprint_objid(var_buf, sizeof(var_buf), vars->name, vars->name_length);
												 fprintf(stderr, "Error: OID not increasing: %s >= %s\n", name_buf, var_buf);
												 running = 0;
												 bExitval = false;
											 }
										memmove((char *)name, (char *)vars->name,
										vars->name_length * sizeof(oid));
										name_length = vars->name_length;
									}
								else
									/* an exception value, so stop */
									running = 0;
							}
						}
				  else
						{ /* error in response, print it */
							running = 0;
							if(response->errstat == SNMP_ERR_NOSUCHNAME)
								{ KMessageBox::sorry(0, i18n("End of MIB\n"));
								}
							else
							 { fprintf(stderr, "Error in packet.\nReason: %s\n",
								 snmp_errstring(response->errstat));
								 if(response->errindex != 0)
									 { QString strErr;
										 
										 for(count = 1, vars = response->variables; vars && count != response->errindex; vars = vars->next_variable, count++)
										 /*EMPTY*/;

										 if(vars)
											 { char szBuf[20];
											   snprint_objid(szBuf, sizeof(szBuf), vars->name, vars->name_length);
											   strErr += szBuf;
											 }
												 
										 KMessageBox::sorry(0, i18n("Failed object: ") + strErr);
									}
								bExitval = false;
							}
					}
      }
    else
			if(status == STAT_TIMEOUT)
				{KMessageBox::sorry(0, i18n("Timeout: No Response from %s\n"), m_Session.peername);
					running = 0;
					bExitval = false;
				}
			else
				{ /* status == STAT_ERROR */
					snmp_sess_perror("SNMPBrowser", m_pSession);
					running = 0;
					bExitval = false;
				}
			if(response)
				snmp_free_pdu(response);
    }

	if(numprinted == 0 && status == STAT_SUCCESS)
		{ /* no printed successful results, which may mean we were
         pointed at an only existing instance.  Attempt a GET, just
         for get measure. */
      QString strVariable;
			SnmpGet(root, rootlen, strVariable);
			strListOfPaths.append(strVariable);
		}

  
	return bExitval;
}

//-----------------------------------------------------------------------------
//
// SnmpFolder implementation
//
//-----------------------------------------------------------------------------

SnmpFolder::SnmpFolder(QListViewItem *parent, const QString& name)
	: SnmpTreeItem(parent, name)
{
	// Set pixmap for SNMP session
	setPixmap(0, *g_pPixmapFolder);
}

SnmpFolder::~SnmpFolder()
{
	
}

/** Return path for this item as string */
QString SnmpFolder::path(const SnmpTreeItem *pItemCaller) const
{
	QString strPath = SnmpTreeItem::path(pItemCaller);

	// folder name
	strPath += text(0);

	return strPath;
}

//-----------------------------------------------------------------------------
//
// SnmpVariable implementation
//
//-----------------------------------------------------------------------------

SnmpVariable::SnmpVariable(QListViewItem *parent, const QString& name)
	: SnmpTreeItem(parent, name)
{
	// Set pixmap for SNMP session
	setPixmap(0, *g_pPixmapVariable);
}

SnmpVariable::~SnmpVariable()
{

}

/** Return path for this item as string */
QString SnmpVariable::path(const SnmpTreeItem *pItemCaller) const
{
	QString strPath = SnmpTreeItem::path(pItemCaller);

	// folder name
	strPath += text(0);

	return strPath;
}

/** Read OID */
void SnmpVariable::Read_objid(const QString &strPath)
{
	m_OidItem.Read_objid(strPath);
}


