/***************************************************************************
                          snmpsession.h  -  description
                             -------------------
    begin                : P ?ec 12 2002
    copyright            : (C) 2002 by Lumir Vanek
    email                : lvanek@users.sourceforge.net 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SNMPSESSION_H
#define SNMPSESSION_H

#include <qlistview.h>
#include <qstringlist.h>

// include files for Net-SNMP
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>

#include "oiditem.h"

class KListView;
class QPixmap;
class QString;

/** Base Class for all tree items
  *@author Lumir Vanek
  */

class SnmpTreeItem : public QListViewItem
{
public:

	SnmpTreeItem(KListView *, const QString&);
	SnmpTreeItem(QListViewItem *, const QString&);


// All possible types of tree items
enum enItemType
{
	itemSession, itemFolder, itemVariable
};

	/**  Return type of item. Must be redefined by derived classes */
	virtual enItemType itemType() const = 0;

	/** Return path for this item as string */
  virtual QString path(const SnmpTreeItem *) const;

  /** Search item fith path from parameter */
  SnmpTreeItem *searchPath(const QString &);

  /** Expand tree node up to level specified in parameter. /-1: expand entire node/.  */
  void expandNode(int = -1, int = 1);

  /** Collapse tree node from level specified in parameter. /-1: collapse entire node/.  */
  void collapseNode(int = -1, int = 1);

protected:
  /** Extract one item name from full path */
  QString getItemNameFromPath(QString &);
};

/**Class for SNMP session with device
  *@author Lumir Vanek
  */

class SnmpSession : public SnmpTreeItem
{
public: 
	SnmpSession(KListView *parent, const QString&, const QString&, const QString&, int);
	~SnmpSession();
  /** Open SNMP session */
  bool SnmpOpen();
  
  /** Get */
  bool SnmpGet(oid *theoid, size_t theoid_len, QString &strVariable);
  
  /**  Return type of tree item */
	enItemType itemType() const { return itemSession; }

  /** Source borrowed from snmpwalk.c, fill string list with output */
  bool SnmpWalk(QStringList &);

  /** Return path for this item as string */
  QString path(const SnmpTreeItem *) const;

protected:
	QCString m_strHost;
	unsigned int m_iPort;
	QCString m_strCommunity;

	struct snmp_session m_Session, *m_pSession;
};

/**Class for SNMP folder in tree
  *@author Lumir Vanek
  */

class SnmpFolder : public SnmpTreeItem
{
public:
	SnmpFolder(QListViewItem *parent, const QString&);
	~SnmpFolder();

  /**  Return type of tree item */
	enItemType itemType() const { return itemFolder; }

	/** Return path for this item as string */
  QString path(const SnmpTreeItem *) const;
};

/**Class for SNMP folder in tree
  *@author Lumir Vanek
  */

#include "oiditem.h"
  
class SnmpVariable : public SnmpTreeItem
{
public:
	SnmpVariable(QListViewItem *parent, const QString&);
	~SnmpVariable();

  /**  Return type of tree item */
	enItemType itemType() const { return itemVariable; }

	/** Return path for this item as string */
  QString path(const SnmpTreeItem *) const;

	/** Read OID */
  void Read_objid(const QString &);
	
	OidItem m_OidItem;
};

#endif
